%define anolis_release 2
Summary: A set of basic GNU tools commonly used in shell scripts
Name:    coreutils
Version: 9.4
Release: %{anolis_release}%{?dist}
License: GPLv3+
Url:     https://www.gnu.org/software/coreutils/
Source0: https://ftp.gnu.org/gnu/%{name}/%{name}-%{version}.tar.xz
Source50:   supported_utils
Source51:   coreutils-provides.inc
Source105:  coreutils-colorls.sh
Source106:  coreutils-colorls.csh
Source301:  sm3sum.sh
Source302:  sm3sum.csh

# do not make coreutils-single depend on /usr/bin/coreutils
%global __requires_exclude ^%{_bindir}/coreutils$

Conflicts: filesystem < 3
# To avoid clobbering installs
Conflicts: coreutils-single

BuildRequires: attr
BuildRequires: autoconf
BuildRequires: automake
BuildRequires: gcc
BuildRequires: gettext-devel
BuildRequires: gmp-devel
BuildRequires: hostname
BuildRequires: libacl-devel
BuildRequires: libattr-devel
BuildRequires: libcap-devel
BuildRequires: libselinux-devel
BuildRequires: libselinux-utils
BuildRequires: make
BuildRequires: openssl-devel
%ifnarch loongarch64
BuildRequires: strace
%endif
BuildRequires: texinfo

# test-only dependencies
BuildRequires: perl-interpreter
BuildRequires: perl(FileHandle)

Requires: %{name}-common = %{version}-%{release}

Provides: coreutils-full = %{version}-%{release}
%include %{SOURCE51}
Obsoletes: %{name} < 8.24

# backport patch from upstream

%description
These are the GNU core utilities.  This package is the combination of
the old GNU fileutils, sh-utils, and textutils packages.

%package single
Summary:  coreutils multicall binary
Suggests: coreutils-common
Provides: coreutils = %{version}-%{release}
%include %{SOURCE51}
# To avoid clobbering installs
Conflicts: coreutils < 8.24
RemovePathPostfixes: .single
%description single
These are the GNU core utilities,
packaged as a single multicall binary.


%package common
Obsoletes: %{name} < 8.24
Suggests: glibc-doc

Summary:  coreutils common optional components
%description common
Optional though recommended components,
including documentation and translations.

%package doc
Summary:        Documentation files for %{name}
Requires:       %{name}-common = %{version}-%{release}
BuildArch:      noarch

%description    doc
The %{name}-doc package contains documentation files for %{name}.

%prep
%autosetup -n %{name}-%{version} -p1

# will be regenerated in the build directories
rm -f src/fs.h

sed src/dircolors.hin \
        -e 's| 00;36$| 01;36|' \
        > DIR_COLORS
sed src/dircolors.hin \
        -e 's| 01;31$| 00;31|' \
        -e 's| 01;35$| 00;35|' \
        > DIR_COLORS.lightbgcolor

(echo ">>> Fixing permissions on tests") 2>/dev/null
find tests -name '*.sh' -perm 0644 -print -exec chmod 0755 '{}' '+'
(echo "<<< done") 2>/dev/null
%ifarch loongarch64
sed -i "/inotify-dir-recreate.sh/d" tests/local.mk
%endif

%build
autoreconf -fiv
export CFLAGS="$RPM_OPT_FLAGS -fno-strict-aliasing -fpic"

# Upstream suggests to build with -Dlint for static analyzers:
# https://lists.gnu.org/archive/html/coreutils/2018-06/msg00110.html
# ... and even for production binary RPMs:
# https://lists.gnu.org/archive/html/bug-gnulib/2020-05/msg00130.html
# There is currently no measurable performance drop or other known downside.
CFLAGS="$CFLAGS -Dlint"

# make mknod work again in chroot without /proc being mounted (#1811038)
export ac_cv_func_lchmod="no"

# needed for out-of-tree build
%global _configure ../configure

%{expand:%%global optflags %{optflags} -D_GNU_SOURCE=1}
for type in separate single; do
  mkdir -p $type && \
  (cd $type || exit $?
  if test $type = 'single'; then
    config_single='--enable-single-binary'
    config_single="$config_single --without-openssl"  # smaller/slower sha*sum
    config_single="$config_single --without-gmp"      # expr/factor machine ints
  else
    config_single='--with-openssl'  # faster sha*sum
  fi
  %configure $config_single \
             --cache-file=../config.cache \
             --enable-install-program=arch \
             --enable-no-install-program=kill,uptime \
             --with-tty-group \
             DEFAULT_POSIX2_VERSION=200112 alternative=199209 || :
  %make_build all V=1

  # make sure that parse-datetime.{c,y} ends up in debuginfo (#1555079)
  ln -fv ../lib/parse-datetime.{c,y} .
  )
done

# Get the list of supported utilities
cp %SOURCE50 .

%check
for type in separate single; do
  test $type = 'single' && subdirs='SUBDIRS=.' # Only check gnulib once
  (cd $type && make check %{?_smp_mflags} $subdirs)
done

%install
for type in separate single; do
  install=install
  if test $type = 'single'; then
    subdir=%{_libexecdir}/%{name}; install=install-exec
  fi
  (cd $type && make DESTDIR=$RPM_BUILD_ROOT/$subdir $install)

  # chroot was in /usr/sbin :
  mkdir -p $RPM_BUILD_ROOT/$subdir/{%{_bindir},%{_sbindir}}
  mv $RPM_BUILD_ROOT/$subdir/{%_bindir,%_sbindir}/chroot

  # Move multicall variants to *.single.
  # RemovePathPostfixes will strip that later.
  if test $type = 'single'; then
    for dir in %{_bindir} %{_sbindir} %{_libexecdir}/%{name}; do
      for bin in $RPM_BUILD_ROOT/%{_libexecdir}/%{name}/$dir/*; do
        basebin=$(basename $bin)
        mv $bin $RPM_BUILD_ROOT/$dir/$basebin.single
      done
    done
  fi
done

mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/profile.d
install -p -c -m644 DIR_COLORS{,.lightbgcolor} $RPM_BUILD_ROOT%{_sysconfdir}
install -p -c -m644 %SOURCE105 $RPM_BUILD_ROOT%{_sysconfdir}/profile.d/colorls.sh
install -p -c -m644 %SOURCE106 $RPM_BUILD_ROOT%{_sysconfdir}/profile.d/colorls.csh
install -p -c -m644 %SOURCE301 $RPM_BUILD_ROOT%{_sysconfdir}/profile.d/sm3sum.sh
install -p -c -m644 %SOURCE302 $RPM_BUILD_ROOT%{_sysconfdir}/profile.d/sm3sum.csh

%find_lang %name
# Add the %%lang(xyz) ownership for the LC_TIME dirs as well...
grep LC_TIME %name.lang | cut -d'/' -f1-6 | sed -e 's/) /) %%dir /g' >>%name.lang

# (sb) Deal with Installed (but unpackaged) file(s) found
rm -f $RPM_BUILD_ROOT%{_infodir}/dir

%generate_compatibility_deps

%files -f supported_utils
%dir %{abidir}
%exclude %{_bindir}/*.single
%dir %{_libexecdir}/coreutils
%{_libexecdir}/coreutils/*.so
%{abidir}/*-option.list

%files single
%{_bindir}/*.single
%{_sbindir}/chroot.single
%dir %{_libexecdir}/coreutils
%{_libexecdir}/coreutils/*.so.single
# duplicate the license because coreutils-common does not need to be installed
%{!?_licensedir:%global license %%doc}
%license COPYING

%files common -f %{name}.lang
%config(noreplace) %{_sysconfdir}/DIR_COLORS*
%config(noreplace) %{_sysconfdir}/profile.d/*
%{_infodir}/coreutils*
%{_mandir}/man*/*
# The following go to /usr/share/doc/coreutils-common
%doc ABOUT-NLS
%license COPYING

%files doc
%doc NEWS README THANKS TODO

%changelog
* Tue Nov  7 2023 Wenlong Zhang<zhangwenlong@loongson.cn> - 9.4-2
- fix build error for coreutils

* Thu Aug 31 2023 Funda Wang <fundawang@yeah.net> - 9.4-1
- New version 9.4

* Mon Aug 7 2023 Funda Wang <fundawang@yeah.net> - 9.3-2
- Add alias for sm3sum

* Wed Apr 19 2023 Funda Wang <fundawang@yeah.net> - 9.3-1
- New version 9.3

* Tue Mar 21 2023 Funda Wang <fundawang@yeah.net> - 9.2-1
- New version 9.2

* Mon Dec 05 2022 Kaihao Bai <carlo.bai@alibaba-inc.com> - 9.1-3
- add upstream bugfix

* Tue Nov 01 2022 mgb01105731 <mgb01105731@alibaba-inc.com> -9.1-2
- optimise spec file

* Wed Sep 21 2022 mgb01105731 <mgb01105731@alibaba-inc.com> -9.1-1
- update to 9.1

* Mon Mar 14 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 9.0-2
- chmod: fix exit status when ignoring symlinks

* Fri Mar 04 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 9.0-1
- init from upstream version 9.0
